#include <stdio.h>
#include <stdlib.h>
#include <glpk.h>


void printResults(glp_prob *lp){	
	int i;
	double j,z = glp_get_obj_val(lp);
	printf("maflow = %g\n",z);
	
	for(i=1;i<=glp_get_num_cols(lp);i++){
		j = glp_get_col_prim(lp,i);
		if(j!=0)
			printf("%4d : %5f\n",i,j);
	}
}

/**
 * Inserts the columns that represent the edge flows, in the solver and the 
 * corresponding constraints that restrict each flow to be positive and smaller 
 * than the capacity of the edge
 */
void setStructuralVariables(glp_prob * lp, int numEdges, float *capacities){
	int i;	
	
	//Add as many cols as the number of edges+1
	glp_add_cols(lp,numEdges+1);
		
	for(i=1;i<=numEdges;i++){
		//glp_set_col_name(lp,i,"f"); //optionally set a symbolic name
		glp_set_col_bnds(lp,i,GLP_DB,-capacities[i-1],capacities[i-1]); //Undirected graph bounds
		//glp_set_col_bnds(lp,i,GLP_DB,0.0,capacities[i-1]); //Directed graph bounds
		glp_set_obj_coef(lp,i,0.0);
	}
	
	//Add col at the end that represents total flow from s to t and is a free variable
	glp_set_col_bnds(lp, numEdges+1, GLP_FR, 0.0, 0.0);
	//That column is at the same time the objective function to maximize
	glp_set_obj_coef(lp, numEdges+1, 1.0);
}

/** 
 * Inserts the auxiliary variables that represent flow conservation at
 * each vertex
 */
void setAuxilliaryVariables(glp_prob * lp, int numVertices){
	int i;
	//We add as many aux variable as the vertices
	glp_add_rows(lp,numVertices);
	
	//Each aux variable represents flow conservation at each edge
	for(i=1;i<=numVertices;i++){
		//glp_set_row_name(lp, i, "ei"); //optionally set symbolic name
		glp_set_row_bnds(lp, i, GLP_UP, 0.0, 0.0);	
	}
	
}

/**
 * Initiates the constraint matrix according to the adjacency matrix,
 * the goal and the target.
 */
void setProblemMatrix(glp_prob * lp, int numVertices, int numEdges, int **Adj, int s, int t){
	int i,j;
	int ind[(numEdges+1)+1];
	double val[(numEdges+1)+1];
	
	for(i=1;i<=numVertices;i++){
		for(j=1;j<=numEdges;j++){			
			ind[j] = j;
			val[j] = Adj[i-1][j-1];
		}
		ind[j] = j;
		if(i==s)
			val[j] = -1;
		else if(i==t)
			val[j] = 1;
		else
			val[j] = 0; 
		glp_set_mat_row(lp, i, numEdges+1, ind, val);
	}	
}

/**
 * Computes the max flow between two vertices.
 * 
 * @param numVertices. The # of vertices
 * @param numEdge. The # of edges
 * @param Adj. The arc-node matrix. (vertices-rows, edges-cols)
 * @param s. The starting vertex
 * @param t. The goal vertex
 */ 
glp_prob* maxFlow(int numVertices, int numEdges, int **Adj, float *capacities, int s, int t){
	//Initialize linear program
	glp_prob *lp;	
	lp = glp_create_prob();
	glp_set_prob_name(lp, "max flow");
	glp_set_obj_dir(lp, GLP_MAX);
	
	//Prepare the problem
	setStructuralVariables(lp,numEdges,capacities);
	setAuxilliaryVariables(lp,numVertices);
	setProblemMatrix(lp,numVertices,numEdges,Adj,s,t);
	
	//Execute simplex
	glp_simplex(lp, NULL);
	
	//printResults(lp);
	
	//Delete problem structure
	//glp_delete_prob(lp);
	return lp;
}
